Summary:    MPI over InfiniBand, Omni-Path, Ethernet/iWARP, RoCE, and Slingshot
Name:       mvapich2
Version:    2.3.7
Release:    2%{?dist}
License:    BSD and MIT
URL:        http://mvapich.cse.ohio-state.edu
Source0:    http://mvapich.cse.ohio-state.edu/download/mvapich/mv2/%{name}-%{version}.tar.gz
Source1:    mvapich2.module.in
Source2:    mvapich2.macros.in

Patch3000:  0001-mvapich23-unbundle-contrib-hwloc-and-osu_benchmarks.patch
Patch5000:  0001-Enhance-condition-check-for-GCC-version.patch

BuildRequires: gcc-gfortran gcc-c++ autoconf automake libtool
BuildRequires: bison flex rdma-core-devel libibumad-devel libibverbs-devel librdmacm-devel
BuildRequires: python3-devel perl-Digest-MD5 hwloc-devel rpm-mpi-hooks infiniband-diags-devel
%ifarch x86_64
BuildRequires: libpsm2-devel
%endif
Requires:      environment-modules
Provides:      mpi


%description
The MVAPICH2 software, based on MPI 3.1 standard, delivers the
best performance, scalability and fault tolerance for high-end
computing systems and servers using InfiniBand, Omni-Path,
Ethernet/iWARP, RoCE(v1/v2), Cray Slingshot 10 and 11, and
Rockport Networks networking technologies.


%package devel
Summary:   Development files for %{name}
Provides:  mpi-devel
Requires:  librdmacm-devel libibverbs-devel libibumad-devel
Requires:  %{name} = %{version}-%{release}
Requires:  gcc-gfortran

%description devel
Contains development headers and libraries for %{name}.

%package doc
Summary:   Documentation files for mvapich2
BuildArch: noarch

%description doc
Additional documentation for mvapich2.

%ifarch x86_64
%package psm2
Summary:   OSU MVAPICH2 MPI package 2.3 for Omni-Path adapters
Requires:  environment-modules
Provides:  mpi

%description psm2
MVAPICH2 is a Message Passing Interface (MPI 3.0) implementation
based on MPICH and developed by Ohio State University. This is
a version of mvapich2 2.3 that uses the PSM2 transport for Omni-Path
adapters.

%package psm2-devel
Summary:   Development files for %{name}-psm2
Requires:  librdmacm-devel libibverbs-devel libibumad-devel
Requires:  %{name}-psm2 = %{version}-%{release}
Requires:  gcc-gfortran
Provides:  mpi-devel

%description psm2-devel
Contains development headers and libraries for %{name}-psm2.
%endif

%prep
%autosetup -p1
rm -r contrib/
rm -r limic2-0.5.6/
rm -r osu_benchmarks/

find . -name configure -exec \
    sed -i -r 's/(hardcode_into_libs)=.*$/\1=no/' '{}' ';'

mkdir .default
mv * .default
mv .default default

%ifarch x86_64
cp -pr default psm2
%endif


%build
export FFLAGS="$FFLAGS -fallow-argument-mismatch"
export AR=ar

%ifarch x86_64
%global variant %{name}-psm2
%global libname %{variant}
%global namearch %{variant}-%{_arch}

cd psm2
./configure \
    --prefix=%{_libdir}/%{libname} \
    --exec-prefix=%{_libdir}/%{libname} \
    --bindir=%{_libdir}/%{libname}/bin \
    --sbindir=%{_libdir}/%{libname}/bin \
    --libdir=%{_libdir}/%{libname}/lib \
    --mandir=%{_mandir}/%{namearch} \
    --includedir=%{_includedir}/%{namearch} \
    --sysconfdir=%{_sysconfdir}/%{namearch} \
    --datarootdir=%{_datadir}/%{libname} \
    --docdir=%{_docdir}/%{name} \
    --enable-error-checking=runtime \
    --enable-timing=none \
    --enable-g=mem,dbg,meminit \
    --enable-fast=all \
    --enable-shared \
    --enable-static \
    --enable-fortran=all \
    --enable-cxx \
    --disable-silent-rules \
    --disable-wrapper-rpath \
    --with-hwloc=v2 \
    --with-device=ch3:psm

find . -name libtool -exec \
    sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g;
            s|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' '{}' ';'
%make_build
cd ..
%endif

%global variant %{name}
%global libname %{variant}
%global namearch %{variant}-%{_arch}

cd default
./configure \
    --prefix=%{_libdir}/%{libname} \
    --exec-prefix=%{_libdir}/%{libname} \
    --bindir=%{_libdir}/%{libname}/bin \
    --sbindir=%{_libdir}/%{libname}/bin \
    --libdir=%{_libdir}/%{libname}/lib \
    --mandir=%{_mandir}/%{namearch} \
    --includedir=%{_includedir}/%{namearch} \
    --sysconfdir=%{_sysconfdir}/%{namearch} \
    --datarootdir=%{_datadir}/%{libname} \
    --docdir=%{_docdir}/%{name} \
    --enable-error-checking=runtime \
    --enable-timing=none \
    --enable-g=mem,dbg,meminit \
    --enable-fast=all \
    --enable-shared \
    --enable-static \
    --enable-fortran=all \
    --enable-cxx \
    --disable-silent-rules \
    --disable-wrapper-rpath \
    --with-hwloc=v2

find . -name libtool -exec \
    sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g;
            s|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' '{}' ';'
%make_build
cd ..


%install
finish_install() {
	local VARIANT="$1"
	local LIBNAME="$VARIANT"
	local NAMEARCH="$VARIANT-%{_arch}"

	local MODULE_TEMPLATE="$2"
	local MACROS_TEMPLATE="$3"
	local FORTRAN_SUBDIR_SUFFIX="$4"

	find %{buildroot}%{_libdir}/$LIBNAME/lib -name \*.la -delete

	mkdir -p %{buildroot}%{_mandir}/$NAMEARCH/man{2,4,5,6,7,8,9,n}
	mkdir -p %{buildroot}/%{_fmoddir}/$LIBNAME$FORTRAN_SUBDIR_SUFFIX
	mkdir -p %{buildroot}/%{python3_sitearch}/$LIBNAME

	mkdir -p %{buildroot}%{_sysconfdir}/modulefiles/mpi
	sed "s#@LIBDIR@#%{_libdir}/$LIBNAME#g;
	     s#@ETCDIR@#%{_sysconfdir}/$NAMEARCH#g;
	     s#@FMODDIR@#%{_fmoddir}/$LIBNAME$FORTRAN_SUBDIR_SUFFIX#g;
	     s#@INCDIR@#%{_includedir}/$NAMEARCH#g;
	     s#@MANDIR@#%{_mandir}/$NAMEARCH#g;
	     s#@PYSITEARCH@#%{python3_sitearch}/$LIBNAME#g;
	     s#@COMPILER@#$NAMEARCH#g;
	     s#@SUFFIX@#_$VARIANT#g" \
		< $MODULE_TEMPLATE \
		> %{buildroot}%{_sysconfdir}/modulefiles/mpi/$NAMEARCH

	mkdir -p %{buildroot}%{_sysconfdir}/rpm
	sed "s#@MACRONAME@#${LIBNAME//[-.]/_}#g;
	     s#@MODULENAME@#$NAMEARCH#" \
		< $MACROS_TEMPLATE \
		> %{buildroot}/%{_sysconfdir}/rpm/macros.$NAMEARCH
}

mkdir -p %{buildroot}%{_docdir}/%{name}

%ifarch x86_64
cd psm2
%make_install
finish_install %{name}-psm2 %SOURCE1 %SOURCE2 ""
cd ..
%endif

cd default
%make_install
finish_install %{name} %SOURCE1 %SOURCE2 ""
cd ..

%global variant %{name}
%global libname %{variant}
%global namearch %{variant}-%{_arch}

%files
%dir %{_libdir}/%{libname}
%dir %{_libdir}/%{libname}/bin
%{_libdir}/%{libname}/bin/hydra_nameserver
%{_libdir}/%{libname}/bin/hydra_persist
%{_libdir}/%{libname}/bin/hydra_pmi_proxy
%{_libdir}/%{libname}/bin/mpichversion
%{_libdir}/%{libname}/bin/mpiexec
%{_libdir}/%{libname}/bin/mpiexec.hydra
%{_libdir}/%{libname}/bin/mpiexec.mpirun_rsh
%{_libdir}/%{libname}/bin/mpiname
%{_libdir}/%{libname}/bin/mpirun
%{_libdir}/%{libname}/bin/mpirun_rsh
%{_libdir}/%{libname}/bin/mpispawn
%{_libdir}/%{libname}/bin/mpivars
%{_libdir}/%{libname}/bin/parkill
%dir %{_libdir}/%{libname}/lib
%{_libdir}/%{libname}/lib/*.so.*
%dir %{_fmoddir}/%{libname}
%dir %{python3_sitearch}/%{libname}
%{_sysconfdir}/modulefiles/mpi/%{namearch}
%dir %{_mandir}/%{namearch}
%dir %{_mandir}/%{namearch}/man*
%{_mandir}/%{namearch}/man1/hydra_*
%{_mandir}/%{namearch}/man1/mpiexec.*

%files devel
%dir %{_includedir}/%{namearch}
%{_includedir}/%{namearch}/*
%{_libdir}/%{libname}/bin/mpic++
%{_libdir}/%{libname}/bin/mpicc
%{_libdir}/%{libname}/bin/mpicxx
%{_libdir}/%{libname}/bin/mpif77
%{_libdir}/%{libname}/bin/mpif90
%{_libdir}/%{libname}/bin/mpifort
%{_libdir}/%{libname}/lib/pkgconfig
%{_libdir}/%{libname}/lib/*.a
%{_libdir}/%{libname}/lib/*.so
%{_sysconfdir}/rpm/macros.%{namearch}
%{_mandir}/%{namearch}/man1/mpi[cf]*
%{_mandir}/%{namearch}/man3/*

%files doc
%{_docdir}/%{name}

%ifarch x86_64
%global variant %{name}-psm2
%global libname %{variant}
%global namearch %{variant}-%{_arch}

%files psm2
%dir %{_libdir}/%{libname}
%dir %{_libdir}/%{libname}/bin
%{_libdir}/%{libname}/bin/hydra_nameserver
%{_libdir}/%{libname}/bin/hydra_persist
%{_libdir}/%{libname}/bin/hydra_pmi_proxy
%{_libdir}/%{libname}/bin/mpichversion
%{_libdir}/%{libname}/bin/mpiexec
%{_libdir}/%{libname}/bin/mpiexec.hydra
%{_libdir}/%{libname}/bin/mpiexec.mpirun_rsh
%{_libdir}/%{libname}/bin/mpiname
%{_libdir}/%{libname}/bin/mpirun
%{_libdir}/%{libname}/bin/mpirun_rsh
%{_libdir}/%{libname}/bin/mpispawn
%{_libdir}/%{libname}/bin/mpivars
%{_libdir}/%{libname}/bin/parkill
%dir %{_libdir}/%{libname}/lib
%{_libdir}/%{libname}/lib/*.so.*
%dir %{_fmoddir}/%{libname}
%dir %{python3_sitearch}/%{libname}
%{_sysconfdir}/modulefiles/mpi/%{namearch}
%dir %{_mandir}/%{namearch}
%dir %{_mandir}/%{namearch}/man*
%{_mandir}/%{namearch}/man1/hydra_*
%{_mandir}/%{namearch}/man1/mpiexec.*

%files psm2-devel
%dir %{_includedir}/%{namearch}
%{_includedir}/%{namearch}/*
%{_libdir}/%{libname}/bin/mpic++
%{_libdir}/%{libname}/bin/mpicc
%{_libdir}/%{libname}/bin/mpicxx
%{_libdir}/%{libname}/bin/mpif77
%{_libdir}/%{libname}/bin/mpif90
%{_libdir}/%{libname}/bin/mpifort
%{_libdir}/%{libname}/lib/pkgconfig
%{_libdir}/%{libname}/lib/*.a
%{_libdir}/%{libname}/lib/*.so
%{_sysconfdir}/rpm/macros.%{namearch}
%{_mandir}/%{namearch}/man1/mpi[cf]*
%{_mandir}/%{namearch}/man3/*
%endif


%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.3.7-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 08 2023 Miaojun Dong <zoedong@tencent.com> - 2.3.7-1
- Bump version to 2.3.7
- Enhance condition check for GCC version

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.3.6-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.3.6-2
- Rebuilt for OpenCloudOS Stream 23

* Fri Mar 17 2023 Miaojun Dong <zoedong@tencent.com> - 2.3.6-1
- initial build
